module CYCSRS.Encoder where
import Data.List
import Data.Maybe
import CYCSRS.Types
import CYCSRS.Encoding
import CYCSRS.Parser

import System.FilePath
import System.Environment

import TPDB.Input
import TPDB.Data (lhs,rhs,rules,name)
        
        
-- runEncode is a very simple interface
runEncoder = do
  args <- getArgs
  if length args < 3 then 
     putStrLn $ "error: you must provide extactly three or more parameters, [srs/trs] [" ++ showEncodings ++ "] <Input>"
   else do 
         let n = length args
         let enc = [getEncoding (args!!i) | i <- [1..n-2]]
         let format = (readFormat $ args!!0)
         if all isJust enc && isJust format then do
           encodeAndWrite (map fromJust enc)  (fromJust format) (args!!(n-1))
           return ()
          else putStrLn $ "error: you must provide extactly three or more parameters, [srs/trs] [" ++ showEncodings ++ "] <Input>"
           

-- ======================================================
-- Reading of Termination Problems and Encoding
-- ======================================================

-- encodeFile gets 
--  * the list of applied encodings, encodings are applied from left to right
--  * the filepath to the input
--  and returns the encoded SRS 
-- note that encodeFile can also be used for reading an SRS without encoding
-- by calling encodeFile NONE file

 
encodeFile :: [Encoding] -> FilePath -> IO SRS
encodeFile encoding file = 
  let (prefix,ext) = splitExtension file 
      e = foldr (.) id (map (\e -> fromJust $ lookup e encodings) (reverse encoding))
  in do
                      srs <- readFileOrXML file
                      return (e srs)

encodeAndWrite :: [Encoding] -> Format -> FilePath -> IO String
encodeAndWrite encoding format infile =
  let (prefix,ext) = splitExtension infile 
  in do
      srs <- encodeFile encoding infile
      let out = formatSRS format srs
      let outfile = (prefix ++ concatMap encodingInfix (encoding) ++ "." ++ formatSuffix format)
      if infile == outfile then return outfile
                           else do
                                 writeFile outfile out
                                 return outfile


encodeAndWriteOutfile :: [Encoding] -> Format -> FilePath -> FilePath -> IO ()
encodeAndWriteOutfile encoding format infile outfile =
  let (prefix,ext) = splitExtension infile 
  in do
      srs <- encodeFile encoding infile
      let out = formatSRS format srs
      if infile == outfile then return ()
                           else do
                                 writeFile outfile out
                                 return ()
                      
                      
-- ========================================================
-- Output formatting
-- ========================================================
                      
data Format = SRS | TRS | XML
 deriving(Show)

readFormat "srs" = Just SRS
readFormat "SRS" = Just SRS
readFormat "TRS" = Just TRS
readFormat "trs" = Just TRS
readFormat "XML" = Just XML
readFormat "xml" = Just XML
readFormat _     = Nothing

formatSuffix SRS = "srs"
formatSuffix TRS = "trs"
formatSuffix XML = "xml"
formatSuffix _   = error "in formatSuffix"


 
-- formatSRS formats the SRS in the given format as a string
formatSRS :: Format -> SRS -> String 
formatSRS format srs = 
  case format of
    SRS -> showSRS srs
    TRS -> showTRS (srsToTRS srs (Var "X")) "X"  
    XML -> error "feature not implemented yet..."
    _   -> error "unknown format"
    
    
-- ============================================    
-- Input processing 
-- ============================================
    
    
readFileOrXML :: FilePath -> IO SRS
readFileOrXML file = 
    let (prefix,ext) = splitExtension file 
    in if  ext == ".xml" then do
          x <- getXML file
          return x 
                         else do
          content <- readFile file
          return (parseSRS content)
          
          
getXML :: FilePath -> IO SRS
getXML file = 
 do
  x <- get_srs file
  return $ map (\x -> (map name (lhs x),map name (rhs x))) $ rules x    