module CYCSRS.Encoding.Shift(shift,shift2,shift3) where

import CYCSRS.Encoding.Tools
import CYCSRS.Types
import Data.List

-- The encoding shift:     
    
{-
B -> W M^k V                            -- 1
M  -> eps                               -- 2
M V a -> V a'                           -- 3
a' b -> b a'                            -- 4
a' E -> a E                             -- 5
W V -> R L                              -- 6
L a -> a'' L                            -- 7
L l -> D r  for all l,r                 -- 8
a'' D -> D a                            -- 9
R D -> B                                -- 10
-}    


shift3 :: CRS -> SRS 
shift3 crs = 
  let alph = nub $ concat $ concatMap (\(a,b) -> [a,b]) crs
      maxnum   = maximum $ map (\(a,b) -> length a) crs
      fresh 1 = zip alph alph
      fresh 2 = freshI 'X' (map snd $ fresh 1) alph
      fresh 3 = freshI 'Y' ((map snd $ fresh 1) ++ (map snd $ fresh 2)) alph
      fresh 0 = freshI 'Z' (((((map snd $ fresh 1) ++ (map snd $ fresh 2)) ++ (map snd $ fresh 3)))) 
        (["B","W","M","V","R","L","D","E"]) 
        
      get i [] = error "<internal error>: get called with []"
      get i s = 
               case (lookup s $ fresh i) of
                Nothing -> (error $ "<internal error>: did not find fresh symbol for: " ++ (show s))
                Just x  -> x
  in     
   [([get 0 "B"], (get 0 "W":(replicate (maxnum-1) (get 0 "M"))) ++ [get 0 "V"])]
   ++
   [([get 0 "M"], [])]
   ++
   [([get 0 "M", get 0 "V", get 1 a], [get 0 "V", get 2 a]) | a <- alph]
   ++
   [([get 2 a, get 1 b], [get 1 b, get 2 a]) | a <- alph, b <- alph]
   ++
   [([get 2 a, get 0 "E"], [get 1 a, get 0 "E"]) | a <- alph]
   ++
   [([get 0 "W", get 0 "V"], [get 0 "R", get 0 "L"])]
   ++
   [([get 0 "L", get 1 a], [get 3 a, get 0 "L"]) | a <- alph]
   ++ 
   [((get 0 "L"):(map (get 1) l), (get 0 "D"):(map (get 1) r)) | (l,r) <- crs]
   ++
   [([get 3 a, get 0 "D"], [get 0 "D", get 1 a]) | a <- alph]
   ++
   [([get 0 "R", get 0 "D"],[get 0 "B"])]
 

shift :: CRS -> SRS 
shift crs = 
 let 
       alph = nub $ concat $ concatMap (\(a,b) -> [a,b]) crs
       maxnum   = maximum $ map (\(a,b) -> length a) crs
       freshsym = fresh alph (["B","M","E"] ++ ["R" ++  a | a <- alph])
       get [] = error "fresh with []"
       get s = case (lookup s freshsym) of
                Nothing -> (error $ "<internal error>: did not find fresh symbol for: " ++ (show s))
                Just x  -> x
 in
  [(l, replicate (maxnum-1) (get "M") ++ r) | (l,r) <- crs]
  ++
  [([a, get "M"], [get "M", a]) | a <- alph]
  ++
  [([get "M"],[])]
  ++
  [([get "B"] ++ replicate k (get "M") ++ [a], [get "B"] ++ replicate (k-1) (get "M") ++ [ra] ) | a <- alph, k <- [1..maxnum], let ra = get ("R" ++ a)]
  ++
  [([ra, b], [b, ra]) | a <- alph, b <- alph, let ra = get ("R"++ a)]
  ++
  [([ra, get "E"], [a, get "E"]) | a <- alph,  let ra = get ("R"++ a)]  
  

  
shift2 :: CRS -> SRS 
shift2 crs = 
 let 
       alph = nub $ concat $ concatMap (\(a,b) -> [a,b]) crs
       maxnum   = maximum $ map (\(a,b) -> length a) crs
       freshsym = fresh alph (["B","M","E","W","R","L","S"])
       get [] = error "fresh with []"
       get s = case (lookup s freshsym) of
                Nothing -> (error $ "<internal error>: did not find fresh symbol for: " ++ (show s))
                Just x  -> x
 in
  [([get "L"] ++  l,  [get "R"] ++ r) | (l,r) <- crs]
  ++
  [([get "B",a], [get "W", get "L"] ++ ((replicate (maxnum-1) "M")) ++ [a]) | a <- alph]
  ++
  [([get "L", get "M"], [get "L"])]
  ++
  ([([get "M", a],[get "S",a]) | a <- alph])
  ++ 
  ([([get "S", a, b],[b, get "S",a]) | a <- alph, b <- alph])
  ++ 
  [([get "S", get "E"],[get "E"])]
  ++ 
  ([([get "L", a],[a, get "L"]) | a <- alph])
  ++ 
  ([([a,get "R"],[get "R",a]) | a <- alph])
  ++ 
  [([get "W",get "R"],[get "B"])]
   