module CYCSRS.ProveParam where
import Data.List
import Data.Char
import System.IO.Unsafe
import CYCSRS.Types
import CYCSRS.Encoding
import CYCSRS.Parser
import CYCSRS.CallsParam
import CYCSRS.Encoder
import System.FilePath
import System.Environment
import System.Process
import Data.Time
import Control.Concurrent
import System.Directory
import System.Console.GetOpt
import Data.Maybe
import TPDB.Input
import TPDB.Data (lhs,rhs,rules,name)
import System.IO 
import System.Exit




        
getXML :: FilePath -> IO SRS
getXML file = 
 do
  x <- get_srs file
  return $ map (\x -> (map name (lhs x),map name (rhs x))) $ rules x
  
getTempFile =
 do
  let name = "tmp_tempfile"
  p <- getPermissions "."
  if writable p then return name
                else error "cannot create temporary files in the current directory, please check permissions"
                
getOutFile = 
 do
  let name = "tmp_outfile"
  p <- getPermissions "."
  if writable p then return name
                else error "cannot create temporary files in the current directory, please check permissions"

getSRSFile = 
 do
  time <- getCurrentTime     
  let name = "tmp_srsfile.srs"
  p <- getPermissions "."
  if writable p then return name
                else error "cannot create temporary files in the current directory, please check permissions"
  
getLOGFile = 
 do
  time <- getCurrentTime     
  let name = "tmp_log"
  p <- getPermissions "."
  if writable p then return name
                else error "cannot create temporary files in the current directory, please check permissions"
    

getSRSFile1 = 
 do
  time <- getCurrentTime     
  let name = "tmp_srsfile1.srs"
  p <- getPermissions "."
  if writable p then return name
                else error "cannot create temporary files in the current directory, please check permissions"

getSRSFile2 = 
 do
  time <- getCurrentTime     
  let name = "tmp_srsfile2.srs"
  p <- getPermissions "."
  if writable p then return name
                else error "cannot create temporary files in the current directory, please check permissions"

getSRSFile3 = 
 do
  time <- getCurrentTime     
  let name = "tmp_srsfile3.srs"
  p <- getPermissions "."
  if writable p then return name
                else error "cannot create temporary files in the current directory, please check permissions"                


  
safeRemoveFile file =
 do
    x <- doesFileExist file
    if x then removeFile file else return ()
                   
data Tool a = APROVE | TORPACYC | COMBINED | SND a | SNDT a | THIRDT a | TSNDT a | THIRD a | IDENTITY | COMB3 | TSND a | TTT2 | COMBINEDT | COMB3T

readTool "comb3"     = Just COMB3
readTool "aprove"    = Just APROVE
readTool "torpacyc"  = Just TORPACYC
readTool "combined"  = Just COMBINED
readTool "combinedt"  = Just COMBINEDT
readTool "comb3t"  = Just COMB3T
readTool "identity"  = Just IDENTITY
readTool "ttt2"  = Just TTT2
readTool "combA1" = Just COMBINED
readTool "combT1" = Just COMBINEDT
readTool "combA2" = Just COMB3
readTool "combT2" = Just COMB3T
readTool _ = Nothing


-- OUTPUTOPT
data OUTPUTOPT = TABBED | SIMPLE | LONG
 deriving(Eq)
getOutputOpt "tabbed" = Just TABBED
getOutputOpt "TABBED" = Just TABBED
getOutputOpt "SIMPLE" = Just SIMPLE
getOutputOpt "simple" = Just SIMPLE
getOutputOpt "long" = Just LONG
getOutputOpt "LONG" = Just LONG
getOutputOpt _ = Nothing


run tool outputopt enc file timeout 
 |  LONG == outputopt = 
    do 
     cleanTmpFiles    
     con <- readFile file
     putStrLn "Input Problem"
     putStrLn "============="     
     putStrLn con
     putStrLn "Transformed Problem"     
     putStrLn "===================="
     srs <- encodeFile enc file
     putStrLn $ showSRS srs
     encodeAndProve tool outputopt enc file timeout
     cleanTmpFiles 
 | otherwise =
    do 
     cleanTmpFiles
     encodeAndProve tool outputopt enc file timeout
     cleanTmpFiles
  
cleanTmpFiles =
 sequence_ [safeRemoveFile file | file <- ["tmp_tempfile"
                                          , "tmp_outfile"
                                          , "tmp_srsfile.srs"
                                          , "tmp_srsfile1.srs"
                                          , "tmp_srsfile2.srs"
                                          , "tmp_srsfile3.srs"
                                          , "param"
                                          , "r"
                                          , "comp.conf"
                                          , "frm"
                                          , "srs"
                                          , "err.out"
                                          , "tmp_log"] ++ allScripts]

runTool  a b c d = 
 do
  f <- getLOGFile
  g <- getTempFile
  fl <- doesFileExist g
  if fl then (system ( "cat " ++ g ++ " >> " ++ f)) >> return () else return ()
  f <- getLOGFile
  appendFile f "****************************************\n" 
  appendFile f ("*** " ++ thetool b  ++ "\n")
  appendFile f "****************************************\n" 
  runIt a b c d

  
  
runIt outputopt APROVE file    maybetimeout  = runAprove outputopt file maybetimeout
runIt outputopt TTT2 file  maybetimeout      = runTTT2 outputopt file maybetimeout
runIt outputopt TORPACYC file  maybetimeout  = runtorpacyc2 outputopt file maybetimeout
runIt outputopt COMBINED file  maybetimeout  = runcombined outputopt file maybetimeout
runIt outputopt COMB3 file  maybetimeout     = runcombined3 outputopt file maybetimeout
runIt outputopt COMBINEDT file  maybetimeout  = runcombinedt outputopt file maybetimeout
runIt outputopt COMB3T file  maybetimeout     = runcombined3t outputopt file maybetimeout
runIt outputopt (SND d)    file    maybetimeout = runAproveSnd outputopt d file maybetimeout
runIt outputopt (THIRD d)    file    maybetimeout = runAproveThird outputopt d file maybetimeout
runIt outputopt (TSND d)    file    maybetimeout = runcombined3' outputopt d file maybetimeout
runIt outputopt (SNDT d)    file    maybetimeout = runTTT2Snd outputopt d file maybetimeout
runIt outputopt (THIRDT d)    file    maybetimeout = runTTT2Third outputopt d file maybetimeout
runIt outputopt (TSNDT d)    file    maybetimeout = runcombined3t' outputopt d file maybetimeout



  
  
thetool  APROVE = "AProVE starts"
thetool  TTT2   = "TTT2 starts "
thetool  TORPACYC = "torpacyc 2015 starts"
thetool  COMBINED = "torpacyc 2015 starts"
thetool  COMB3 = "AProVE starts"
thetool  COMBINEDT = "torpacyc 2015 starts"
thetool  COMB3T = "TTT2 starts"
thetool  (SND d) = "AProVE starts"
thetool  (THIRD d) = "AProVE starts"
thetool  (TSND d)  = "torpacyc 2015 starts"
thetool  (SNDT d)  = "TTT2 015 starts"
thetool  (THIRDT d) = "TTT2 starts"
thetool  (TSNDT d) = "torpacyc 2015 starts"


     
encodeAndProve COMBINED outputopt enc file maybetimeout = 
   do
     newfile1 <- getSRSFile
     newfile2 <- getSRSFile1
     encodeAndWriteOutfile enc SRS file newfile1
     encodeAndWriteOutfile [NONE] SRS newfile1 newfile2
     runTool outputopt COMBINED newfile2 maybetimeout 

     
encodeAndProve COMBINEDT outputopt enc file maybetimeout = 
   do 
     newfile1 <- getSRSFile
     newfile2 <- getSRSFile1
     encodeAndWriteOutfile enc SRS file newfile1
     encodeAndWriteOutfile [NONE] SRS newfile1 newfile2
     runTool outputopt COMBINEDT newfile2 maybetimeout      

encodeAndProve COMB3 outputopt enc file maybetimeout = 
   do
   
     newfile1 <- getSRSFile
     encodeAndWriteOutfile enc SRS file newfile1
     runTool outputopt COMB3 newfile1 maybetimeout 
     
encodeAndProve IDENTITY outputopt enc file maybetimeout = encodeAndWrite enc SRS file >> putStrLn "OK"

encodeAndProve tool outputopt enc file maybetimeout = 
   do
     newfile1 <- getSRSFile
     encodeAndWriteOutfile enc SRS file newfile1
     runTool outputopt tool newfile1 maybetimeout 
       
 
allScripts = [aprove,aproveA,torpacyc,ttt2,ttt2A]
aprove    = "./call_aprove.sh"
aproveA    = "./call_aproveA.sh"
torpacyc  = "./call_torpacyc.sh"

ttt2      = "./call_ttt2.sh"
ttt2A     = "./call_ttt2A.sh"


pretty  TABBED cmd answer time 
  = putStrLn $ "COMMAND: \t" ++ cmd ++ "\tANSWER:\t" ++ answer ++ "\tTIME:\t" ++ show time

pretty  SIMPLE _ answer _ 
 | "YES" `isInfixOf` answer = putStrLn "Yes"
 | "NO" `isInfixOf` answer = putStrLn "No"
 | otherwise = putStrLn "Maybe"

pretty LONG cmd answer time =
  do
  f <- getTempFile
  g <- getLOGFile
  system $ "cat " ++ f ++ " >> " ++ g
  con <- readFile g
  putStrLn "Proof (as delivered by provers):"
  putStrLn "==========="    
  putStrLn con
  putStrLn "Statistics:"
  putStrLn "==========="  
  putStrLn $ "TIME:\t" ++ show time
  putStrLn $ "ANSWER:\t" ++ answer  
 
 
runAproveSnd outputopt d file maybetimeout = 
   do
     removeERRFile
     time <- getCurrentTime     
     tmpfile <- getTempFile
     resultfile <- getOutFile
     removeOUTFile tmpfile
     let cmd = aprove ++ " " ++ file ++ " > " ++ tmpfile -- ++ " 2> err.out"
     b <- getCurrentTime  
     x <- runCommand  cmd
     ecode <- waitForProcess x
     e <- getCurrentTime  
     ok <- doesFileExist tmpfile
     if ok then do 
         c <- readFile  tmpfile
         let answer = (head2 (lines c) ecode)
         writeFile resultfile answer
         pretty outputopt cmd answer (d + (utctDayTime e- utctDayTime b))
           else do
                 pretty outputopt cmd "CRASHED" (d + (utctDayTime e- utctDayTime b))
                 writeFile resultfile "CRASHED"

                 
runAproveThird outputopt d file maybetimeout = 
   do
     removeERRFile
     time <- getCurrentTime     
     tmpfile <- getTempFile
     resultfile <- getOutFile
     removeOUTFile tmpfile
     let cmd = aprove ++ " " ++ file ++ " > " ++ tmpfile -- ++ " 2> err.out"
     b <- getCurrentTime  
     x <- runCommand  cmd
     ecode <- waitForProcess x
     e <- getCurrentTime  
     ok <- doesFileExist tmpfile
     if ok then do 
         c <- readFile  tmpfile
         let answer = (head2 (lines c) ecode)
         writeFile resultfile answer
         pretty outputopt cmd answer (d + (utctDayTime e- utctDayTime b))
           else do
                 pretty outputopt cmd "CRASHED" (d + (utctDayTime e- utctDayTime b))
                 writeFile resultfile "CRASHED"
                 
runAprove outputopt file maybetimeout = 
   do
     removeERRFile
     mkCallAprove maybetimeout
     time <- getCurrentTime     
     tmpfile <- getTempFile
     resultfile <- getOutFile
     removeOUTFile tmpfile        
     let cmd = aprove ++ " " ++ file ++ " > " ++ tmpfile -- ++ " 2> err.out"
     b <- getCurrentTime  
     x <- runCommand  cmd
     ecode <- waitForProcess x
     e <- getCurrentTime  
     ok <- doesFileExist tmpfile
     if ok then do 
         c <- readFile tmpfile
         let answer = (head2 (lines c) ecode)
         writeFile resultfile answer
         pretty outputopt cmd  answer ((utctDayTime e- utctDayTime b))
         
           else do
                 pretty outputopt cmd "CRASHED" ((utctDayTime e- utctDayTime b))
                 writeFile resultfile "CRASHED"


                 
runtorpacyc2 outputopt file maybetimeout  = 
   do
     removeSRSFile      
     mkCallTorpacyc maybetimeout
     time <- getCurrentTime
     tmpfile <- getTempFile
     resultfile <- getOutFile
     removeOUTFile tmpfile        
     let cmd = torpacyc ++ " < " ++ file ++ " > " ++ tmpfile -- ++ " 2> err.out"
     b <- getCurrentTime  
     x <- runCommand  cmd
     ecode <- waitForProcess x
     e <- getCurrentTime 
--     con <- readFile "err.out"
--     crashed <- if "value" `isInfixOf` con && "range" `isInfixOf` con then return True else return False
     ok <- doesFileExist "srs"
     if ok then do 
                       c <- readFile "srs"
                       let answer = if "YES" `isPrefixOf` c then "YES" else  (checkExitcode ecode)
                       writeFile resultfile answer
                       pretty outputopt cmd answer  ((utctDayTime e- utctDayTime b))

                       
           else do
                 pretty outputopt cmd "CRASHED" ((utctDayTime e- utctDayTime b))                 
                 writeFile resultfile "CRASHED"                 

removeSRSFile = 
  do
    x <- doesFileExist "srs"
    if x then removeFile "srs" else return ()

removeERRFile    =
  do
    x <- doesFileExist "err.out"
    if x then removeFile "err.out" else return ()
    system "touch err.out"
    
removeOUTFile file   =
  do
    x <- doesFileExist file
    if x then removeFile file  else return ()
    

    
runcombined outputopt file maybetimeout = 
   do
     removeERRFile
     removeSRSFile      
     mkCallCombined  maybetimeout
     time <- getCurrentTime
     tmpfile <- getTempFile
     resultfile <- getOutFile
     removeOUTFile tmpfile        
     let cmd = torpacyc ++ " < " ++ file ++ " > " ++ tmpfile -- ++ " 2> err.out"
     b <- getCurrentTime  
     x <- runCommand  cmd
     ecode <- waitForProcess x
     e <- getCurrentTime 
--     con <- readFile "err.out"
--     crashed <- if "value" `isInfixOf` con && "range" `isInfixOf` con then return True else return False
     ok <- doesFileExist "srs"
     if ok then do 
                       c <- readFile "srs"
                       let answer = if "YES" `isPrefixOf` c then "YES" else  (checkExitcode ecode)
                       if answer == "YES" then  do
                        pretty outputopt cmd answer  ((utctDayTime e- utctDayTime b))
                        writeFile resultfile answer
                        else
                         do
                          srsfile <- getSRSFile2
                          system $ "cp srs " ++ srsfile
                          encodeAndProve  (SND ((utctDayTime e- utctDayTime b))) outputopt [SPLIT] (srsfile) maybetimeout
                       
           else do
                  srsfile <- getSRSFile2
                  system $ "cp srs " ++ srsfile                 
                  encodeAndProve  (SND ((utctDayTime e- utctDayTime b))) outputopt [SPLIT] (srsfile) maybetimeout

                 
                 
parseEncodeAndShowSRS e input = showSRS (e (parseSRS input))
 
parseEncodeAndShowTRS e input = showTRS (srsToTRS (e (parseSRS input)) (Var "X")) "X"       
 

head2 [] ecode     = 
  case ecode of
     (ExitFailure 124) -> "TIMEOUT"
     _                 -> "MAYBE"
head2 (x:xs) ecode = filter (\x -> not (x `elem` ['\r','\n']))  $ x

mlast [] = "CRASH"
mlast xs = filter (\x -> not (x `elem` ['\r','\n']))  $ last xs

checkTorpacyc []  ecode = 
  case ecode of
     (ExitFailure 124) -> "TIMEOUT"
     _                 -> "MAYBE"
checkTorpacyc str ecode =
  if ("termination has been proved" `isInfixOf` (last $ lines str))
    || ("Terminating due to match bound") `isInfixOf` str
   then "YES"
   else
          case ecode of
            (ExitFailure 124) -> "TIMEOUT"
            _                 -> "MAYBE"

checkExitcode ecode = 
    case ecode of
     (ExitFailure 124) -> "TIMEOUT"
     _                 -> "MAYBE"
     
     
dropSuffix suf str
  | suf `isSuffixOf` str = reverse $ drop (length suf) $ reverse str
  | otherwise = str


manipulate f = do
 system $ "echo " ++ hdr ++ " >> .tmphtml && cat " ++ f ++ " >> .tmphtml && echo " ++ ft ++ ">> .tmphtml && cp .tmphtml " ++ f ++ " && rm .tmphtml"
 return ()

hdr = "\"<html><body><xmp>\""
ft  = "\"</xmp></body></html>\""


runcombined3 outputopt file maybetimeout = 
   do
     removeERRFile
     removeSRSFile      
     mkCallCombined3 maybetimeout
     time <- getCurrentTime
     tmpfile <- getTempFile
     resultfile <- getOutFile
     removeOUTFile tmpfile        
     -- let cmd = torpacyc2 ++ " < " ++ file ++ " > " ++ tmpfile -- ++ " 2> err.out"
     let cmd = aproveA ++ " " ++ file ++ " > " ++ tmpfile -- ++ " 2> err.out"     
     b <- getCurrentTime       
     x <- runCommand  cmd
     ecode <- waitForProcess x
     e <- getCurrentTime  
     ok <- doesFileExist tmpfile
     if ok then do 
         c <- readFile tmpfile
         let answer = (head2 (lines c) ecode)
         if "NO" `isPrefixOf` answer then do
             writeFile resultfile answer
             pretty outputopt cmd  answer ((utctDayTime e- utctDayTime b))
                                     else do
                                            srsfile <- getSRSFile1
                                            system $ "cp " ++ file ++ " " ++ srsfile
                                            encodeAndProve (TSND ((utctDayTime e- utctDayTime b))) outputopt [NONE] srsfile maybetimeout
           else do
                 pretty outputopt cmd "CRASHED" ((utctDayTime e- utctDayTime b))
                 writeFile resultfile "CRASHED"
                 


    
runcombined3' outputopt d file maybetimeout = 
 do
  removeERRFile
  removeSRSFile      
  time <- getCurrentTime
  tmpfile <- getTempFile
  resultfile <- getOutFile
  removeOUTFile tmpfile        
  let cmd = torpacyc ++ " < " ++ file ++ " > " ++ tmpfile -- ++ " 2> err.out"
  b <- getCurrentTime  
  x <- runCommand  cmd
  ecode <- waitForProcess x
  e <- getCurrentTime 
--  con <- readFile "err.out"
--  crashed <- if "value" `isInfixOf` con && "range" `isInfixOf` con then return True else return False
  ok <- doesFileExist "srs"
  if ok then do 
              c <- readFile "srs"
              let answer =  if "YES" `isPrefixOf` c then "YES" else (checkExitcode ecode)
              if answer == "YES" then  do
                  pretty outputopt cmd answer  (d+(utctDayTime e- utctDayTime b))
                  writeFile resultfile answer
                else do 
                        srsfile <- getSRSFile2
                        system $ "cp srs " ++ srsfile
                        encodeAndProve  (THIRD (d+(utctDayTime e- utctDayTime b))) outputopt [SPLIT] (srsfile) maybetimeout
           else do
                 srsfile <- getSRSFile2
                 system $ "cp " ++ file ++ " " ++ srsfile
                 encodeAndProve  (THIRD ((d+utctDayTime e- utctDayTime b))) outputopt [SPLIT] (srsfile) maybetimeout
                 -- putStrLn $ "\tANSWER:\tCRASHED" ++ "\tTIME:\t" ++ show ((utctDayTime e- utctDayTime b))
                 
                 
runTTT2 outputopt file maybetimeout = 
   do
     removeERRFile
     mkCallTTT2 maybetimeout
     time <- getCurrentTime     
     tmpfile <- getTempFile
     resultfile <- getOutFile
     removeOUTFile tmpfile        
     let cmd = ttt2 ++ " " ++ file ++ " > " ++ tmpfile -- ++ " 2> err.out"
     b <- getCurrentTime  
     x <- runCommand  cmd
     ecode <- waitForProcess x
     e <- getCurrentTime  
     ok <- doesFileExist tmpfile
     if ok then do 
         c <- readFile tmpfile
         let answer = (head2 (lines c) ecode)
         writeFile resultfile answer
         pretty outputopt cmd  answer ((utctDayTime e- utctDayTime b))
         
           else do
                 pretty outputopt cmd "CRASHED" ((utctDayTime e- utctDayTime b))
                 writeFile resultfile "CRASHED"
                 

                 
                 
runcombinedt outputopt file maybetimeout = 
   do
     removeERRFile
     removeSRSFile      
     mkCallCombinedt  maybetimeout
     time <- getCurrentTime
     tmpfile <- getTempFile
     resultfile <- getOutFile
     removeOUTFile tmpfile        
     let cmd = torpacyc ++ " < " ++ file ++ " > " ++ tmpfile -- ++ " 2> err.out"
     b <- getCurrentTime  
     x <- runCommand  cmd
     ecode <- waitForProcess x
     e <- getCurrentTime 
     -- con <- readFile "err.out"
     -- crashed <- if "value" `isInfixOf` con && "range" `isInfixOf` con then return True else return False
     ok <- doesFileExist "srs"
     if ok then do 
                       c <- readFile "srs"
                       let answer =  if "YES" `isPrefixOf` c then "YES" else (checkExitcode ecode)
                       if answer == "YES" then  do
                        pretty outputopt cmd answer  ((utctDayTime e- utctDayTime b))
                        writeFile resultfile answer
                        else
                         do
                          srsfile <- getSRSFile2
                          system $ "cp srs " ++ srsfile                         
                          encodeAndProve  (SNDT ((utctDayTime e- utctDayTime b))) outputopt [SPLIT] (srsfile) maybetimeout
                      
                        
                        
                        
                       
           else do
                 srsfile <- getSRSFile2
                 system $ "cp srs " ++ srsfile      
                 encodeAndProve  (SNDT ((utctDayTime e- utctDayTime b))) outputopt [SPLIT] (srsfile) maybetimeout
                 -- putStrLn $ "\tANSWER:\tCRASHED" ++ "\tTIME:\t" ++ show ((utctDayTime e- utctDayTime b))                 
                 
                 
                 
                 
                 
                 
                 
                 
                 
runTTT2Snd outputopt d file maybetimeout = 
   do
     removeERRFile
     time <- getCurrentTime     
     tmpfile <- getTempFile
     resultfile <- getOutFile
     removeOUTFile tmpfile
     let cmd = ttt2 ++ " " ++ file ++ " > " ++ tmpfile -- ++ " 2> err.out"
     b <- getCurrentTime  
     x <- runCommand  cmd
     ecode <- waitForProcess x
     e <- getCurrentTime  
     ok <- doesFileExist tmpfile
     if ok then do 
         c <- readFile  tmpfile
         let answer = (head2 (lines c) ecode)
         writeFile resultfile answer
         pretty outputopt cmd answer (d + (utctDayTime e- utctDayTime b))
           else do
                 pretty outputopt cmd "CRASHED" (d + (utctDayTime e- utctDayTime b))
                 writeFile resultfile "CRASHED"

                 
runTTT2Third outputopt d file maybetimeout = 
   do
     removeERRFile
     time <- getCurrentTime     
     tmpfile <- getTempFile
     resultfile <- getOutFile
     removeOUTFile tmpfile
     let cmd = ttt2 ++ " " ++ file ++ " > " ++ tmpfile -- ++ " 2> err.out"
     b <- getCurrentTime  
     x <- runCommand  cmd
     ecode <- waitForProcess x
     e <- getCurrentTime  
     ok <- doesFileExist tmpfile
     if ok then do 
         c <- readFile  tmpfile
         let answer = (head2 (lines c) ecode)
         writeFile resultfile answer
         pretty outputopt cmd answer (d + (utctDayTime e- utctDayTime b))
           else do
                 pretty outputopt cmd "CRASHED" (d + (utctDayTime e- utctDayTime b))
                 writeFile resultfile "CRASHED"                 
                 
runcombined3t' outputopt d file maybetimeout = 
 do
  removeERRFile
  removeSRSFile      
  time <- getCurrentTime
  tmpfile <- getTempFile
  resultfile <- getOutFile
  removeOUTFile tmpfile        
  let cmd = torpacyc ++ " < " ++ file ++ " > " ++ tmpfile -- ++ " 2> err.out"
  b <- getCurrentTime  
  x <- runCommand  cmd
  ecode <- waitForProcess x
  e <- getCurrentTime 
--  con <- readFile "err.out"
--  crashed <- if "value" `isInfixOf` con && "range" `isInfixOf` con then return True else return False
  ok <- doesFileExist "srs"
  if ok then do 
              c <- readFile "srs"
              let answer = if "YES" `isPrefixOf` c then "YES" else  (checkExitcode ecode)
              if answer == "YES" then  do
                  pretty outputopt cmd answer  (d+(utctDayTime e- utctDayTime b))
                  writeFile resultfile answer
                else do 
                        srsfile <- getSRSFile3
                        system $ "cp srs " ++ srsfile
                        encodeAndProve  (THIRDT (d+(utctDayTime e- utctDayTime b))) outputopt [SPLIT] (srsfile) maybetimeout
           else do
                 srsfile <- getSRSFile3
                 system $ "cp srs " ++ srsfile                 
                 encodeAndProve  (THIRDT ((d+utctDayTime e- utctDayTime b))) outputopt [SPLIT] (srsfile) maybetimeout
                 -- putStrLn $ "\tANSWER:\tCRASHED" ++ "\tTIME:\t" ++ show ((utctDayTime e- utctDayTime b))
                                  

runcombined3t outputopt file maybetimeout = 
   do
     removeERRFile
     removeSRSFile      
     mkCallCombined3t maybetimeout
     time <- getCurrentTime
     tmpfile <- getTempFile
     resultfile <- getOutFile
     removeOUTFile tmpfile        
     let cmd = ttt2A ++ " " ++ file ++ " > " -- ++ tmpfile ++ " 2> err.out"     
     b <- getCurrentTime       
     x <- runCommand  cmd
     ecode <- waitForProcess x
     e <- getCurrentTime  
     ok <- doesFileExist tmpfile
     if ok then do 
         c <- readFile tmpfile
         let answer = (head2 (lines c) ecode)
         if "NO" `isPrefixOf` answer then do
             writeFile resultfile answer
             pretty outputopt cmd  answer ((utctDayTime e- utctDayTime b))
                                     else do
                                            srsfile <- getSRSFile1
                                            system $ "cp " ++ file ++ " " ++ srsfile                                     
                                            encodeAndProve (TSNDT ((utctDayTime e- utctDayTime b))) outputopt [NONE] srsfile maybetimeout
           else do
                 pretty outputopt cmd "CRASHED" ((utctDayTime e- utctDayTime b))
                 writeFile resultfile "CRASHED"                                  
