module CYCSRS.Types where
import Data.List

-- ============================================
-- Data structures
-- ============================================ 

-- type synonyms for symbols and for alphabets. We use Strings as symbols.
type Symbol = String
type Alphabet = [Symbol]
-- a sequence is a list of Symbol:
type Sequence  = [Symbol]
-- a string rewrite rule is a pair of sequences
type Rule    = (Sequence,Sequence )
-- a string rewrite system is a list of rules
type SRS       = [Rule]
-- a cycle rewrite system is represented like a string rewrite system
-- but the rewrite relation is different
type CRS      = SRS 
-- terms, term rewriting systems
data Term = Fn String [Term] | Var String
 deriving(Eq,Show)
type TRSRule = (Term,Term)
type TRS = [TRSRule]

hasEmptyLHS srs = any null (map fst srs)
hasEmptyRHS srs = any null (map snd srs)

sequenceToTerm :: Sequence   -- Input sequence
                     -> Term -- Variable used in the output term
                     -> Term -- Output term
sequenceToTerm [] v     = v
sequenceToTerm (x:xs) v = Fn x [sequenceToTerm xs v]

srsToTRS :: SRS -- Input SRS
                -> Term -- Variable used in the output TRS
                -> TRS  -- Output TRS
                
srsToTRS sequence v = map (\(lhs,rhs) -> (sequenceToTerm lhs v,sequenceToTerm rhs v)) sequence

showSRS :: SRS -> String
showSRS srs = "(RULES \n" ++ showSRS' srs ++ ")"
 where 
  showSRS' [] = []
  showSRS' ((l,r):xs) = (concat $ intersperse " " l) ++ " -> " ++ (concat $ intersperse " " r) ++ ",\n" ++ showSRS' xs

showTerm (Fn str [])   = str
showTerm (Fn str trms) = str ++ "(" ++ concat (intersperse "," (map showTerm trms)) ++ ")"
showTerm (Var v)       = v

showTRS :: TRS -> String -> String
showTRS r var = "(VAR " ++ var ++")\n(RULES\n" ++ showTRS' r ++ "\n)\n"
 where
  showTRS' [] = []
  showTRS' ((l,r):xs) ="   "++ showTerm l ++ " -> " ++ showTerm r ++ "\n" ++ showTRS' xs
  
showSequence xs = concat $ intersperse " " xs  
 
alphabet_size srs = length $ nub $ concat $ concatMap (\(x,y) -> [x,y]) srs   
no_rules srs = length srs
no_symbs srs = length $ concat $ concatMap (\(x,y) -> [x,y]) srs   

 
