module Main where
import Data.List
import Data.Maybe
import CYCSRS.Types
import CYCSRS.Encoding
import CYCSRS.Parser
import CYCSRS.CallsParam
import CYCSRS.Encoder
import CYCSRS.ProveParam
import System.FilePath
import System.Environment
import System.Process
import Data.Time
import Control.Concurrent
import System.Directory
import System.Console.GetOpt
import Data.Maybe
import TPDB.Input
import TPDB.Data (lhs,rhs,rules,name)
import System.IO
import System.Exit

-- Mode of the tool
data Mode = ENCODE | PROVE | STATISTIC | HELP

getMode "encode" = Just ENCODE
getMode "ENCODE" = Just ENCODE
getMode "prove"  = Just PROVE
getMode "PROVE"  = Just PROVE
getMode "statistic" = Just STATISTIC
getMode "STATISTIC" = Just STATISTIC
getMode "help" = Just HELP
getMode "HELP" = Just HELP
getMode _         = Nothing




main = 
 do 
  args <- getArgs
  let len = length args
  if (len == 0) then displayErr "insufficient arguments: no mode given" 
   else do
    let (mode:args') = args
    case getMode mode of
     Nothing -> displayErr $ "invalid arguments: mode " ++ mode ++ " not supported"
     -- ==============================================================================================
     -- Mode: ENCODE 
     --     
     Just ENCODE -> do
       name <- getProgName
       if  len < 4 then displayErr $ "insufficient arguments:\n"++ name ++" encode OUTPUTFORMAT ENCODINGS INFILE [-oOUTFILE]" else do
        let (infile,maybeoutfile) = if "-o" `isPrefixOf` (args!!(len-1)) 
                                      then (args!!(len-2),Just (tail $ tail $ args!!(len-1)))
                                      else ((args!!(len-1)),Nothing)
        let encs = getEncodings [args!!i | i <- if isJust maybeoutfile then [2..len-3] else [2..len-2]]
        let format = readFormat (args!!1)                                    
        infileExists <- doesFileExist infile                    
        let check = isJust encs && isJust format && infileExists
        if check then do
            case maybeoutfile of 
                  Just outfile -> encodeAndWriteOutfile (fromJust encs) (fromJust format) infile outfile
                  Nothing      -> do srs <- (encodeFile (fromJust encs) infile)
                                     putStrLn $ formatSRS (fromJust format) srs
                                    
          else displayErr $ "invalid arguments: " ++ 
                            (if isNothing encs then "invalid encodings" 
                                               else if isNothing format then "invalid format" 
                                                                        else "file: " ++ infile ++ " doesn't exist") ++
                            "\n"++ name ++" encode OUTPUTFORMAT ENCODINGS INFILE [-oOUTFILE]"
     --
     -- ==============================================================================================
     -- Mode: PROVE
     --
     Just PROVE -> do
       name <- getProgName       
       if  len < 5 then displayErr $ "insufficient arguments:\n" ++ name ++" prove  TOOL OUTPUTOPT ENCODINGS INFILE" else do       
        let (infile,maybetimeout) = if "-t" `isPrefixOf` (args!!(len-1)) 
                                      then (args!!(len-2),Just (read $ tail $ tail $ (args!!(len-1))))
                                      else ((args!!(len-1)),Nothing)         
        let outputopt = getOutputOpt (args!!2)
        let encs = getEncodings [args!!i | i <- if isJust maybetimeout then [3..len-3] else [3..len-2]]
        let tool = readTool (args!!1)
        infileExists <- doesFileExist infile        
        let check = isJust encs && isJust tool && isJust outputopt && infileExists
        if check then 
            run (fromJust tool) (fromJust outputopt) (fromJust encs) infile maybetimeout
         else displayErr $ "invalid arguments: " ++
                           (if isNothing encs then "invalid encodings" 
                                             else if isNothing tool then "invalid tool" 
                                                                    else if isNothing outputopt then "invalid output option"
                                                                                                else "file: " ++ infile ++ " doesn't exist") ++
                           "\n" ++ name ++ " prove TOOL OUTPUTOPT ENCODINGS INFILE"
     -- ==============================================================================================
     -- Mode: STATISTIC
     --
     Just STATISTIC -> do
       name <- getProgName       
       if  len < 3 then displayErr $ "insufficient argument:\n" ++ name ++" statistic ENCODINGS INFILE"  else do       
        let infile = (args!!(len-1))
        let encs = getEncodings [args!!i | i <- [1..len-2]]
        infileExists <- doesFileExist infile
        let check = isJust encs && infileExists
        if check then do
          let (prefix,ext) = splitExtension infile 
          srs <- encodeFile (fromJust encs)  infile
          let outname = prefix ++ concatMap encodingInfix (fromJust encs)
          prettyStat srs outname
         else displayErr $ "invalid arguments: " ++
                           (if isNothing encs then "invalid encodings" 
                                              else "file: " ++ infile ++ " doesn't exist") ++
                           "\n" ++ name ++" statistic ENCODINGS INFILE" 
                            
     --
     -- ==============================================================================================
     -- Mode: HELP
     --
     Just HELP -> displayHelp
       
prettyStat srs outname =
  putStrLn $ "File: \t" 
              ++ outname 
              ++ "\tAlphabet:\t"
              ++ show (alphabet_size srs) 
              ++ "\tRules:\t" 
              ++ show (no_rules srs) 
              ++ "\tSymbols:\t" 
              ++ show (no_symbs srs) 
              ++ "\tEmptyRHS:\t" 
              ++ show (hasEmptyRHS srs) 
              ++ "\tEmptyLHS:\t" 
              ++ show (hasEmptyLHS srs)
      

  
 

displayErr err =
  do
    name <- getProgName
    putStrLn $ "error: " ++ err ++ "\ntype '"++ name++ " help' to display all options" 

displayHelp = 
   do 
     name <-getProgName
     putStrLn $ helptext name
     
tbd = putStrLn "feature not implemented yet"


helptext name =
 unlines $
 ["AVAILABLE OPTIONS"
 ," * "++ name ++" encode OUTPUTFORMAT ENCODINGS INFILE [-oOUTFILE]"
 ,"   Encode the string rewrite system in INFILE and print the encoded string"
 ,"   rewrite system on stdout (or write output to OUTFILE if provided by -o)"
 ,"    "
 ," * "++ name ++" prove TOOL OUTPUTOPT ENCODINGS INFILE [-tTIMEOUT]"
 ,"   Prove termination of the encoded string or cycle rewrite relation."
 ,"   An optional timeout in seconds can be set "
 ,"    "
 ," * "++ name ++" statistic ENCODINGS INFILE"
 ,"   Print statistics of the encoded string rewrite system"
 ,"   "
 ," * "++ name ++" help"
 ,"   Show help"
 ,"    "
 ,"ENCODINGS"
 ,""
 ," a sequence of encodings applied to the input from left to right "
 ," (separated by whitespaces, without quotes, e.g. srscyc split )"
 ," available encodings are:"
 ,"  none   = no encoding is applied"
 ,"  split  = encode cycle into string rewriting by splitting the rules"
 ,"  rotate = encode cycle into string rewriting by rotation and prefix rewriting"
 ,"  shift  = encode cycle into string rewriting by shifting and rewriting"
 ,"  srscyc = encode string rewriting into cycle rewriting "
 ,""
 ,"OUTPUTFORMAT"
 ,""
 ," available formats are:"
 ,"  srs    = string rewrite system in text format"
 ,"  trs    = term rewrite system in text format"
 ,""
 ,"TOOL   "
 ,""
 ," select the tool to prove termination"
 ,""
 ,"      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
 ,"      !!! ENSURE that the NECESSARY SOFTWARE is installed and           !!!"
 ,"      !!! ADJUST the pathes to your LOCAL CONFIGURATION in the file     !!!"
 ,"      !!!                                                               !!!"
 ,"      !!!        conf.sh                                                !!!"
 ,"      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
 ,""
 ," available tools are:"
 ,"  torpacyc  = proves cycle termination by matrix interpretations"
 ,"  aprove    = termination prover AProVE"
 ,"  ttt2      = termination prover TTT2" 
 ,"  combA1    = first torpacyc2 and if necessary the encoding split is applied"
 ,"              to the remaining termination problem and then AProVE is called"
 ,"  combA2    = first AProVE is applied to check for cycle nontermination,"
 ,"              then torpacyc2 and then split together with AProVE are applied"
 ,"  combT1    = first torpacyc2 and if necessary the encoding split is applied"
 ,"              to the remaining termination problem and then TTT2 is called"
 ,"  combT2    = first TTT2 is applied to check for cycle nontermination,"
 ,"              then torpacyc2 and then split together with TTT2 are applied" 
 ,"          "          
 ,"OUTPUTOPT"
 ,""
 ,"  simple = only print the answer (yes/no/maybe)"
 ,"  long   = verbose output including input, output of prover asf."
 ,"  tabbed = print the internal command, the answer, the time, separated by tabs"
 ,"      "
 ,"INFILE"
 ," the format of the content is resolved by its file extension"
 ,"  .srs   = string rewrite system in text format"
 ,"  .xml   = string rewrite system in XML format of the TPDB"
 ,"      " 
 ,"TIMEOUT"
 ," a number of seconds"
 ,"      "
 ,"Configuration"
 ,"============="
 ," the available parameters can be adjusted in the file"
 ,"    conf.sh "
 ,""
 ,"Examples"
 ,"========"
 ,"  * "++ name ++" encode trs srscyc split myinfile.srs -omyoutfile        "
 ,"    encodes the SRS in myinfile.srs by applying first the transformation "
 ,"    srscyc and thereafter the transformation split."
 ,"    The generated rewrite system is written as a term rewrite system "
 ,"    into myoutfile"
 ,"        "
 ,"   * "++ name ++" prove torpacyc long none myinfile.xml -t60"
 ,"     proves cycle termination of the SRS in myinfile.xml using torpacyc"
 ,"     with long output and a timeout of 60sec."
 ,"        "
 ,"   * "++ name ++" prove aprove simple split myinfile.srs"
 ,"     proves cycle termination of the SRS in myinfile.srs by first transforming"
 ,"     the system using the split-transformation and then running the prover"
 ,"     prover AProVE on the generated SRS with no timeout"

 
 ]

